﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SpaceFighterGame
{
    public class AnimatedSprite
    {
        #region Fields
        private Texture2D texture { get; set; }
        private Int32 rows { get; set; }
        private Int32 columns { get; set; }
        private Int32 currentFrame;
        private Int32 totalFrames;
        public Boolean finished;
        public Vector2 spriteLocation;
        #endregion

        #region Properties
        public Boolean Finished
        {
            get { return finished; }
            set { finished = value; }
        }

        public Vector2 SpriteLocation
        {
            get { return spriteLocation; }
            set { spriteLocation = value; }
        }
        #endregion

        #region Contructor
        public AnimatedSprite(Texture2D texture, Int32 rows, Int32 columns)
        {
            this.texture = texture;
            this.rows = rows;
            this.columns = columns;
            currentFrame = 0;
            totalFrames = rows * columns;
            finished = false;
        }
        #endregion

        #region Public methods
        /// <summary>
        /// Léptetjük az aktuális sprite frame-t.
        /// </summary>
        public void Update()
        {
            currentFrame++;
            if (currentFrame == totalFrames)
                finished = true;
        }

        /// <summary>
        /// Kirajzoljuka z aktuális spriteframe-t.
        /// </summary>
        public void Draw(SpriteBatch spriteBatch, Vector2 location)
        {
            Int32 width = texture.Width / columns;
            Int32 height = texture.Height / rows;
            Int32 row = (int)((float)currentFrame / (float)columns);
            Int32 column = currentFrame % columns;

            Rectangle sourceRectangle = new Rectangle(width * column, height * row, width, height);
            Rectangle destinationRectangle = new Rectangle((int)location.X, (int)location.Y, width, height);

            spriteBatch.Draw(texture, destinationRectangle, sourceRectangle, Color.White);
        }
        #endregion
    }
}
