﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SpaceFighterGame
{
    public class EnemyManager
    {
        #region Fields
        private List<Enemy> enemies;
        private List<Laser> enemyLasers;
        private ContentManager Content;
        private SpriteBatch spriteBatch;
        #endregion

        #region Properties
        public List<Enemy> Enemies
        {
            get { return enemies; }
            set { enemies = value; }
        }

        public List<Laser> EnemyLasers
        {
            get { return enemyLasers; }
            set { enemyLasers = value; }
        }
        #endregion

        #region Constructor
        public EnemyManager(ContentManager Content, SpriteBatch spriteBatch)
        {
            enemies = new List<Enemy>();
            enemyLasers = new List<Laser>();
            this.Content = Content;
            this.spriteBatch = spriteBatch;
        }
        #endregion

        #region Public methods
        public void SpawnEnemy()
        {
            Enemy newEnemy = new Enemy(Content);
            enemies.Add(newEnemy);
        }

        public void MoveEnemies()
        {
            foreach (var enemy in enemies)
            {
                if (!enemy.Destroyed)
                {
                    enemy.MoveEnemy();
                }
            }

            enemies = enemies.Where(e => !e.Destroyed).ToList();
        }

        public void DrawEnemies()
        {
            foreach (var enemy in enemies)
            {
                if (!enemy.Destroyed)
                    spriteBatch.Draw(enemy.Texture, enemy.GetLocation(), Color.White);
            }
        }

        public void FireLasers()
        {
            foreach (var enemy in enemies)
            {
                lock (enemyLasers)
                {
                    Laser newLaser = new Laser(
                        Content,
                        enemy.LocX + (enemy.SizeX / 2),
                        enemy.LocY + enemy.SizeY,
                        true
                        );
                    enemyLasers.Add(newLaser);
                }
            }
        }

        public void DrawEnemyLasers()
        {
            lock (enemyLasers)
            {
                foreach (var laser in enemyLasers)
                {
                    spriteBatch.Draw(laser.Texture, laser.GetLocation(), Color.White);
                }
            }
        }
        #endregion
    }
}
