using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using System.Timers;

namespace SpaceFighterGame
{
    public class SpaceFighterGame : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        private Texture2D background;
        Fighter fighter;
        Planet planet;
        Timer timer, enemyTimer;
        List<Laser> lasers;
        DateTime laserAdded;
        EnemyManager enemyManager;
        AnimatedSprite animatedSprite;
        Texture2D spriteTexture;

        public SpaceFighterGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            timer = new Timer();
            timer.Interval = 30;
            timer.Elapsed += timer_Elapsed;

            enemyTimer = new Timer();
            enemyTimer.Interval = 4000;
            enemyTimer.Elapsed += enemyTimer_Elapsed;

            lasers = new List<Laser>();
            laserAdded = DateTime.Now;
        }


        /// <summary>
        /// Ellensgeket s lzerjeiket megjelent esemny
        /// </summary>
        void enemyTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            enemyManager.SpawnEnemy();
            enemyManager.FireLasers();
        }

        /// <summary>
        /// Ellensgeket s lzerjeiket mozgat esemny
        /// </summary>
        void timer_Elapsed(object sender, ElapsedEventArgs e)
        {
            planet.MovePlanet();

            foreach (var laser in lasers)
            {
                laser.MoveLaser();
            }

            lasers = lasers.Where(l => !l.ReachedScreenBorder).ToList();

            lock (enemyManager.EnemyLasers)
            {
                foreach (var laser in enemyManager.EnemyLasers)
                {
                    laser.MoveEnemyLaser();
                }

                enemyManager.EnemyLasers = enemyManager.EnemyLasers.Where(l => !l.ReachedScreenBorder).ToList();
            }


            enemyManager.MoveEnemies();
            CheckHit();
        }

        /// <summary>
        /// Inicializl fggvny
        /// </summary>
        protected override void Initialize()
        {
            base.Initialize();
        }

        /// <summary>
        /// Erforrsokat betlt fggvny
        /// </summary>
        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);
            background = Content.Load<Texture2D>("stars");
            spriteTexture = Content.Load<Texture2D>("spritesheet1");
            fighter = new Fighter(Content);
            planet = new Planet(Content);
            enemyManager = new EnemyManager(Content, spriteBatch);
            timer.Start();
            enemyTimer.Start();
        }

        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Frisst mvelet.
        /// Itt kapjuk el a klnbz user inputokat
        /// valamint itt ltpetjk a robbans kirajzolsst.
        /// </summary>
        protected override void Update(GameTime gameTime)
        {

            if (Keyboard.GetState(PlayerIndex.One).IsKeyDown(Keys.Left))
            {
                if (fighter.LocX - 2 > 0)
                    fighter.LocX -= 2;
            }

            if (Keyboard.GetState(PlayerIndex.One).IsKeyDown(Keys.Right))
            {
                if (fighter.LocX + 2 < 800 - fighter.SizeX)
                    fighter.LocX += 2;
            }

            if (Keyboard.GetState(PlayerIndex.One).IsKeyDown(Keys.Up))
            {
                if (fighter.LocY - 2 > 0)
                    fighter.LocY -= 2;
            }

            if (Keyboard.GetState(PlayerIndex.One).IsKeyDown(Keys.Down))
            {
                if (fighter.LocY + 2 < 480 - fighter.SizeY)
                    fighter.LocY += 2;
            }

            if (Keyboard.GetState(PlayerIndex.One).IsKeyDown(Keys.Space))
            {
                DateTime newLaserAdd = DateTime.Now;
                if (newLaserAdd - laserAdded > TimeSpan.FromSeconds(1))
                {
                    laserAdded = DateTime.Now;
                    Laser newLaser = new Laser(Content, fighter.LocX + (fighter.SizeX / 2), fighter.LocY);
                    lasers.Add(newLaser);
                }
            }

            if (Keyboard.GetState(PlayerIndex.One).IsKeyDown(Keys.Back))
                this.Exit();

            if (animatedSprite != null && !animatedSprite.finished)
                animatedSprite.Update();

            base.Update(gameTime);
        }

        /// <summary>
        /// Rajzol fggvny, mely elszr letrli a kpernyt
        /// majd egyesvel felrajzolja a htteret, bolygt, a lzereket,
        /// az ellensgeket, a mi rhajnkat valamint a robbans animcit.
        /// </summary>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();

            spriteBatch.Draw(background, new Rectangle(0, 0, 800, 480), Color.White);
            spriteBatch.Draw(planet.Texture, planet.GetLocation(), Color.White);

            foreach (var laser in lasers)
            {
                spriteBatch.Draw(laser.Texture, laser.GetLocation(), Color.White);
            }

            enemyManager.DrawEnemies();
            enemyManager.DrawEnemyLasers();

            spriteBatch.Draw(fighter.Texture, fighter.GetLocation(), Color.White);

            if (animatedSprite != null && !animatedSprite.finished)
                animatedSprite.Draw(spriteBatch, animatedSprite.spriteLocation);

            spriteBatch.End();

            base.Draw(gameTime);
        }

        /// <summary>
        /// tkzst vizsgl fggvny. Minden egyes lzerre megvizsgljuk,
        /// hogy elrt e valamilyen rhajt.
        /// </summary>
        private void CheckHit()
        {
            foreach (var laser in lasers)
            {
                foreach (var enemy in enemyManager.Enemies)
                {
                    if (laser.LocX >= enemy.LocX && laser.LocX <= enemy.LocX + enemy.SizeX && laser.LocY <= enemy.LocY + enemy.SizeX)
                    {
                        laser.ReachedScreenBorder = true;
                        enemy.Destroyed = true;
                        animatedSprite = new AnimatedSprite(spriteTexture, 9, 9);
                        animatedSprite.spriteLocation = new Vector2(enemy.LocX, enemy.LocY);
                    }
                }
            }
            lasers = lasers.Where(l => !l.ReachedScreenBorder).ToList();
            enemyManager.Enemies = enemyManager.Enemies.Where(e => !e.Destroyed).ToList();

            lock (enemyManager.EnemyLasers)
            {
                foreach (var laser in enemyManager.EnemyLasers)
                {
                    if (laser.LocX >= fighter.LocX && laser.LocX <= fighter.LocX + fighter.SizeX
                        &&
                        laser.LocY >= fighter.LocY && laser.LocY <= fighter.LocY + fighter.SizeX)
                    {
                        laser.ReachedScreenBorder = true;
                        animatedSprite = new AnimatedSprite(spriteTexture, 9, 9);
                        animatedSprite.spriteLocation = new Vector2(fighter.LocX, fighter.LocY);
                        fighter.LocY = fighter.LocX = -50;
                    }
                }
            }

        }
    }
}
